'use strict';

var view = require('@codemirror/view');
var language = require('@codemirror/language');
var highlight = require('@lezer/highlight');
var _utils = require('@utils');

/**
 * Enhanced GitHub Light theme color palette
 * ----------------------------------------
 * Colors organized by function with visual color blocks for reference
 */
// Core UI colors
const base00 = '#ffffff'; // Background (GitHub light mode background)
const base01 = '#24292e'; // Foreground (main text color)
const base02 = '#BBDFFF'; // Selection - light blue
const base03 = '#6e7781'; // Comments, subdued text
const base04 = '#f6f8fa'; // Light gray (gutter, panels)
// Syntax highlighting colors
const base05 = '#116329'; // Tag names - GitHub green
const base06 = '#6a737d'; // Comments, brackets - GitHub gray
const base07 = '#6f42c1'; // Classes, properties - GitHub purple
const base08 = '#005cc5'; // Variables, attributes - GitHub blue
const base09 = '#d73a49'; // Keywords, types - GitHub red
const base0A = '#032f62'; // Strings, regexps - GitHub navy
const base0B = '#22863a'; // Names, quotes - GitHub green
const base0C = '#e36209'; // Atoms, booleans - GitHub orange
// Background variants
const base0D = '#f1f8ff'; // Active line gutter background
const base0E = '#e1e4e8'; // Panel and tooltip border color
const base0F = '#f8f9fa'; // Tooltip background
// Link colors
const linkColor = '#0969da'; // Bright blue for links
const visitedLinkColor = '#8250df'; // Purple for visited links
// Special states
const invalid = '#cb2431'; // Invalid color - error red
const highlightBackground = '#BBDFFF20'; // Line highlight (light blue and opacity)
const tooltipBackground = base0F; // Tooltip background
const cursor = base01; // Caret color
const selection = base02; // Selection color
const activeBracketBg = '#e8f0fe'; // Active bracket background
const activeBracketBorder = '#0366d6'; // Active bracket border
const diagnosticWarning = '#b08800'; // Warning color
const selectionMatch = '#79b8ff40'; // Selection match background
/**
 * Enhanced editor theme styles for GitHub Light
 */
const githubLightTheme = view.EditorView.theme({
    // Base editor styles
    '&': {
        color: base01,
        backgroundColor: base00,
        fontSize: _utils.generalContent.fontSize,
        fontFamily: _utils.generalContent.fontFamily,
    },
    // Content and cursor
    '.cm-content': {
        caretColor: cursor,
        lineHeight: _utils.generalContent.lineHeight,
    },
    '.cm-cursor, .cm-dropCursor': {
        borderLeftColor: cursor,
        borderLeftWidth: _utils.generalCursor.borderLeftWidth,
    },
    '.cm-fat-cursor': {
        backgroundColor: `${cursor}99`,
        color: base00,
    },
    // Selection
    '&.cm-focused > .cm-scroller > .cm-selectionLayer .cm-selectionBackground, .cm-selectionBackground, .cm-content ::selection': {
        backgroundColor: selection,
        color: base01,
    },
    // Make sure selection appears above active line
    '.cm-selectionLayer': {
        zIndex: 100,
    },
    // Search functionality
    '.cm-searchMatch': {
        backgroundColor: '#daebff',
        outline: `1px solid ${base08}`,
        color: base01,
        borderRadius: _utils.generalSearchField.borderRadius,
        '& span': {
            color: base01,
        },
    },
    '.cm-searchMatch.cm-searchMatch-selected': {
        backgroundColor: '#79b8ff',
        color: base00,
        padding: _utils.generalSearchField.padding,
        '& span': {
            color: base00,
        },
    },
    '.cm-search.cm-panel.cm-textfield': {
        color: base01,
        borderRadius: _utils.generalSearchField.borderRadius,
        padding: _utils.generalSearchField.padding,
    },
    // Panels
    '.cm-panels': {
        backgroundColor: base04,
        color: base03,
        borderRadius: '0 0 4px 4px',
    },
    '.cm-panels.cm-panels-top': {
        borderBottom: `1px solid ${base0E}`,
    },
    '.cm-panels.cm-panels-bottom': {
        borderTop: `1px solid ${base0E}`,
    },
    '.cm-panel button': {
        backgroundColor: base00,
        color: base01,
        border: _utils.generalPanel.border,
        borderRadius: _utils.generalPanel.borderRadius,
        padding: _utils.generalPanel.padding,
    },
    '.cm-panel button:hover': {
        backgroundColor: '#f5f5f5',
    },
    // Line highlighting
    '.cm-activeLine': {
        backgroundColor: highlightBackground,
        borderRadius: _utils.generalLine.borderRadius,
        zIndex: 1,
    },
    // Gutters
    '.cm-gutters': {
        backgroundColor: base04,
        color: base03,
        border: _utils.generalGutter.border,
        borderRight: `1px solid ${base0E}`,
        paddingRight: _utils.generalGutter.paddingRight,
    },
    '.cm-activeLineGutter': {
        backgroundColor: base0D,
        color: base01,
        fontWeight: _utils.generalGutter.fontWeight,
    },
    '.cm-lineNumbers': {
        fontSize: _utils.generalGutter.fontSize,
    },
    '.cm-foldGutter': {
        fontSize: _utils.generalGutter.fontSize,
    },
    '.cm-foldGutter .cm-gutterElement': {
        color: base03,
        cursor: 'pointer',
    },
    '.cm-foldGutter .cm-gutterElement:hover': {
        color: base01,
    },
    // Tooltips and autocomplete
    '.cm-tooltip': {
        backgroundColor: tooltipBackground,
        border: `1px solid ${base0E}`,
        borderRadius: _utils.generalTooltip.borderRadius,
        padding: _utils.generalTooltip.padding,
        boxShadow: '0 1px 5px rgba(0, 0, 0, 0.1)',
    },
    '.cm-tooltip-autocomplete': {
        '& > ul': {
            backgroundColor: tooltipBackground,
            border: 'none',
        },
        '& > ul > li': {
            padding: _utils.generalTooltip.padding,
            lineHeight: _utils.generalTooltip.lineHeight,
        },
        '& > ul > li[aria-selected]': {
            backgroundColor: '#0366d630',
            color: base01,
            borderRadius: _utils.generalTooltip.borderRadiusSelected,
        },
        '& > ul > li:hover': {
            backgroundColor: '#0366d615',
        },
        '& > ul > li > span.cm-completionIcon': {
            color: base03,
            paddingRight: _utils.generalTooltip.paddingRight,
        },
        '& > ul > li > span.cm-completionDetail': {
            color: base03,
            fontStyle: 'italic',
        },
    },
    '.cm-tooltip .cm-tooltip-arrow:before': {
        borderTopColor: 'transparent',
        borderBottomColor: 'transparent',
    },
    '.cm-tooltip .cm-tooltip-arrow:after': {
        borderTopColor: tooltipBackground,
        borderBottomColor: tooltipBackground,
    },
    // Diagnostics styling
    '.cm-diagnostic': {
        '&-error': {
            borderLeft: `3px solid ${invalid}`,
        },
        '&-warning': {
            borderLeft: `3px solid ${diagnosticWarning}`,
        },
        '&-info': {
            borderLeft: `3px solid ${linkColor}`,
        },
    },
    '.cm-lintPoint-error': {
        borderBottom: `2px wavy ${invalid}`,
    },
    '.cm-lintPoint-warning': {
        borderBottom: `2px wavy ${diagnosticWarning}`,
    },
    // Matching brackets
    '.cm-matchingBracket': {
        backgroundColor: activeBracketBg,
        outline: `1px solid ${activeBracketBorder}80`,
        borderRadius: _utils.generalMatching.borderRadius,
    },
    '.cm-nonmatchingBracket': {
        backgroundColor: '#ffeef080',
        outline: `1px solid ${invalid}`,
        borderRadius: _utils.generalMatching.borderRadius,
    },
    // Selection matches
    '.cm-selectionMatch': {
        backgroundColor: selectionMatch,
        outline: `1px solid ${base02}50`,
        borderRadius: _utils.generalMatching.borderRadius,
    },
    // Fold placeholder
    '.cm-foldPlaceholder': {
        backgroundColor: selectionMatch,
        color: base03,
        border: `1px dotted ${base0E}70`,
        borderRadius: _utils.generalPlaceholder.borderRadius,
        padding: _utils.generalPlaceholder.padding,
        margin: _utils.generalPlaceholder.margin,
    },
    // Focus outline
    '&.cm-focused': {
        outline: 'none',
        boxShadow: `0 0 0 2px ${base00}, 0 0 0 3px ${linkColor}40`,
    },
    // Scrollbars
    '& .cm-scroller::-webkit-scrollbar': {
        width: _utils.generalScroller.width,
        height: _utils.generalScroller.height,
    },
    '& .cm-scroller::-webkit-scrollbar-track': {
        background: base0E,
    },
    '& .cm-scroller::-webkit-scrollbar-thumb': {
        backgroundColor: base01,
        borderRadius: _utils.generalScroller.borderRadius,
        border: `3px solid ${base0E}`,
    },
    '& .cm-scroller::-webkit-scrollbar-thumb:hover': {
        backgroundColor: base06,
    },
    // Ghost text
    '.cm-ghostText': {
        opacity: '0.5',
        color: '#959da5',
    },
}, { dark: false });
/**
 * Enhanced syntax highlighting for GitHub Light theme
 */
const githubLightHighlightStyle = language.HighlightStyle.define([
    // Keywords and control flow
    { tag: highlight.tags.keyword, color: base09, fontWeight: 'bold' },
    { tag: highlight.tags.controlKeyword, color: base09, fontWeight: 'bold' },
    { tag: highlight.tags.moduleKeyword, color: base09, fontWeight: 'bold' },
    // Names and variables
    { tag: [highlight.tags.name, highlight.tags.deleted, highlight.tags.character, highlight.tags.macroName], color: base08 },
    { tag: [highlight.tags.variableName], color: base08 },
    { tag: [highlight.tags.propertyName], color: base07, fontStyle: 'normal' },
    // Classes and types
    { tag: [highlight.tags.typeName], color: base09 },
    { tag: [highlight.tags.className], color: base07, fontStyle: 'italic' },
    { tag: [highlight.tags.namespace], color: base08, fontStyle: 'italic' },
    // Operators and punctuation
    { tag: [highlight.tags.operator, highlight.tags.operatorKeyword], color: base01 },
    { tag: [highlight.tags.bracket], color: base06 },
    { tag: [highlight.tags.brace], color: base06 },
    { tag: [highlight.tags.punctuation], color: base06 },
    // Functions and parameters
    { tag: [highlight.tags.function(highlight.tags.variableName), highlight.tags.labelName], color: base0B },
    { tag: [highlight.tags.definition(highlight.tags.variableName)], color: base08 },
    // Constants and literals
    { tag: highlight.tags.number, color: base0C },
    { tag: highlight.tags.changed, color: base0C },
    { tag: highlight.tags.annotation, color: invalid, fontStyle: 'italic' },
    { tag: highlight.tags.modifier, color: base0C, fontStyle: 'italic' },
    { tag: highlight.tags.self, color: base0C },
    { tag: [highlight.tags.color, highlight.tags.constant(highlight.tags.name), highlight.tags.standard(highlight.tags.name)], color: base0C },
    { tag: [highlight.tags.atom, highlight.tags.bool, highlight.tags.special(highlight.tags.variableName)], color: base0C },
    // Strings and regex
    { tag: [highlight.tags.processingInstruction, highlight.tags.inserted], color: base0B },
    { tag: [highlight.tags.special(highlight.tags.string), highlight.tags.regexp], color: base0A },
    { tag: highlight.tags.string, color: base0A },
    // Punctuation and structure
    { tag: highlight.tags.definition(highlight.tags.typeName), color: base09, fontWeight: 'bold' },
    // Comments and documentation
    { tag: highlight.tags.meta, color: base06 },
    { tag: highlight.tags.comment, fontStyle: 'italic', color: base06 },
    { tag: highlight.tags.docComment, fontStyle: 'italic', color: base06 },
    // HTML/XML elements
    { tag: [highlight.tags.tagName], color: base05 },
    { tag: [highlight.tags.attributeName], color: base07 },
    // Markdown and text formatting
    { tag: [highlight.tags.heading], fontWeight: 'bold', color: base08 },
    { tag: [highlight.tags.strong], fontWeight: 'bold', color: base08 },
    { tag: [highlight.tags.emphasis], fontStyle: 'italic', color: base0A },
    // Links and URLs
    {
        tag: [highlight.tags.link],
        color: visitedLinkColor,
        fontWeight: '500',
        textDecoration: 'underline',
        textUnderlinePosition: 'under',
    },
    {
        tag: [highlight.tags.url],
        color: linkColor,
        textDecoration: 'underline',
        textUnderlineOffset: '2px',
    },
    // Special states
    {
        tag: [highlight.tags.invalid],
        color: base01,
        textDecoration: 'underline wavy',
        borderBottom: `1px wavy ${invalid}`,
    },
    { tag: [highlight.tags.strikethrough], color: invalid, textDecoration: 'line-through' },
    // Enhanced syntax highlighting
    { tag: highlight.tags.constant(highlight.tags.name), color: base0C },
    { tag: highlight.tags.deleted, color: invalid },
    { tag: highlight.tags.squareBracket, color: base06 },
    { tag: highlight.tags.angleBracket, color: base06 },
    // Additional specific styles
    { tag: highlight.tags.monospace, color: base01 },
    { tag: [highlight.tags.contentSeparator], color: base08 },
    { tag: highlight.tags.quote, color: base06 },
]);
/**
 * Combined GitHub Light theme extension
 */
const githubLight = [
    githubLightTheme,
    language.syntaxHighlighting(githubLightHighlightStyle),
];

exports.githubLight = githubLight;
exports.githubLightHighlightStyle = githubLightHighlightStyle;
exports.githubLightTheme = githubLightTheme;
